# -*- coding: utf-8 -*-
from Plugins.Plugin import PluginDescriptor
from Components.ActionMap import *
from Components.Label import Label
from Screens.MessageBox import MessageBox
from Components.ActionMap import NumberActionMap, ActionMap
from Components.MenuList import MenuList
from Screens.Console import Console
from Components.config import config, ConfigText, ConfigSubsection, configfile
from enigma import gFont, addFont, eTimer, eConsoleAppContainer, gPixmapPtr, ePicLoad, loadPNG, getDesktop, \
    eListboxPythonMultiContent, RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_HALIGN_CENTER, \
    RT_VALIGN_CENTER, eListbox
from Components.MultiContent import MultiContentEntryText
from Screens.Screen import Screen
from Tools.Directories import resolveFilename, SCOPE_LANGUAGE, SCOPE_PLUGINS
from Components.Language import language
import gettext
import os
from bs4 import BeautifulSoup
import urllib2
from twisted.internet import reactor, threads

import ssl
try:
    _create_unverified_https_context = ssl._create_unverified_context
except AttributeError:
    # Legacy Python that doesn't verify HTTPS certificates by default
    pass
else:
    # Handle target environment that doesn't support HTTPS verification
    ssl._create_default_https_context = _create_unverified_https_context


damnPanels = ["GoldenPanel", "SatVenusPanel", "GoldenFeed", "PersianDreambox", "DreamOSatDownloader"]
for damnPanel in damnPanels:
    if os.path.exists("/usr/lib/enigma2/python/Plugins/Extensions/" + damnPanel + "/plugin.pyo"):
        os.remove("/usr/lib/enigma2/python/Plugins/Extensions/" + damnPanel + "/plugin.pyo")
    if os.path.exists("/usr/lib/enigma2/python/Plugins/Extensions/" + damnPanel + "/plugin.py"):
        os.remove("/usr/lib/enigma2/python/Plugins/Extensions/" + damnPanel + "/plugin.py")


config.plugins.wledflash = ConfigSubsection()
config.plugins.wledflash.update = ConfigText(default="", fixed_size=False)

VERSION = "1.0.8"

url = "https://github.com/Aircoookie/WLED/releases/"
url_git = "https://github.com"

lang = language.getLanguage()
os.environ["LANGUAGE"] = lang[:2]
gettext.bindtextdomain("enigma2", resolveFilename(SCOPE_LANGUAGE))
gettext.textdomain("enigma2")
gettext.bindtextdomain("WLED",
                       "%s%s" % (resolveFilename(SCOPE_PLUGINS), "Extensions/WledFlash/locale/"))

EXT_WLED_DESTINATION = "/media/hdd/WLED"


def _(txt):
    t = gettext.dgettext("WLED", txt)
    if t == txt:
        t = gettext.gettext(txt)
    return t


INFO = "Package: enigma2-plugin-extensions-wledflash\nVersion: " + VERSION + _("\nDescription: WLED flash Tool for enigma2\nMaintainer: murxer, pclin <support@boxpirates.to>\n\n\nThe standard binary files apply to WS2812b-compatible stripes.\nFurther stripe types can be found in the file name of the binary files.\nIf you don't find anything for your stripes, you have to compile yourself. See WLED-Wiki for compatible LED stripes and compile information.\n\nThanks to Aircoookie for the great WLED software!")

DESKTOPSIZE = getDesktop(0).size()

if DESKTOPSIZE.width() == 1920:
    skinFactor = 1
else:
    skinFactor = 1.5


class WledFlash(Screen):
    def __init__(self, session):
        if DESKTOPSIZE.width() == 1920:
            self.skin = """
                         <screen name="WLEDFlash" backgroundColor="#00ffffff" position="0,0" size="1920,1080" title="WLED Flash Tool" flags="wfNoBorder">
                         <eLabel name="BackgroundColor" position="2,2" size="1916,1076" zPosition="1" backgroundColor="#002f4665" />
                         
                         <widget name="release" position="35,50" size="395,40" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 28" valign="center" halign="left" />
                         <widget name="releaseList" position="30,100" size="400,900" backgroundColorSelected="#00000000" foregroundColorSelected="#00ffffff" foregroundColor="#00000000" backgroundColor="#00ffffff" scrollbarMode="showOnDemand" zPosition="3" transparent="0" />
                         
                         <widget name="binFile" position="460,50" size="495,40" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 28" valign="center" halign="left" />
                         <widget name="binList" position="455,100" size="600,900" backgroundColorSelected="#00000000" foregroundColorSelected="#00ffffff" foregroundColor="#00000000" backgroundColor="#00ffffff" scrollbarMode="showOnDemand" zPosition="3" transparent="0" />
                         
                         <eLabel name="BackgroundColor" position="1090,100" size="800,900" zPosition="1" backgroundColor="#00ffffff" />
                         <widget name="info" position="1095,105" size="790,850" foregroundColor="#00000000" backgroundColor="#00ffffff" font="Regular; 30" valign="top" halign="left"  zPosition="2" transparent="0" />
                         
                         <eLabel name="red" position="30,1038" size="204,37" zPosition="2" backgroundColor="#00ff0000" />
                         <widget name="reset" position="32,1040" size="200,33" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 24" valign="center" halign="center" />
                        
                         <eLabel text="Info" position="1569,1040" size="107,33" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 24" valign="center" halign="center" />
                         <eLabel text="OK" position="1676,1040" size="107,33" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 24" valign="center" halign="center" />
                         <eLabel text="Exit" position="1783,1040" size="107,33" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 24" valign="center" halign="center" />
                         </screen>
                         """
        else:
            self.skin = """
                         <screen name="WLEDFlash" backgroundColor="#00ffffff" position="0,0" size="1280,720" title="WLED Flash Tool" flags="wfNoBorder">
                         <eLabel name="BackgroundColor" position="1,1" size="1277,717" zPosition="1" backgroundColor="#002f4665" />

                         <widget name="release" position="23,33" size="263,26" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 18" valign="center" halign="left" />
                         <widget name="releaseList" position="20,66" size="266,600" backgroundColorSelected="#00000000" foregroundColorSelected="#00ffffff" foregroundColor="#00000000" backgroundColor="#00ffffff" scrollbarMode="showOnDemand" zPosition="3" transparent="0" />

                         <widget name="binFile" position="306,33" size="330,26" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 18" valign="center" halign="left" />
                         <widget name="binList" position="303,66" size="400,600" backgroundColorSelected="#00000000" foregroundColorSelected="#00ffffff" foregroundColor="#00000000" backgroundColor="#00ffffff" scrollbarMode="showOnDemand" zPosition="3" transparent="0" />

                         <eLabel name="BackgroundColor" position="726,66" size="534,600" zPosition="1" backgroundColor="#00ffffff" />
                         <widget name="info" position="730,70" size="526,576" foregroundColor="#00000000" backgroundColor="#00ffffff" font="Regular; 20" valign="top" halign="left"  zPosition="2" transparent="0" />

                         <eLabel name="red" position="20,692" size="136,24" zPosition="2" backgroundColor="#00ff0000" />
                         <widget name="reset" position="21,693" size="133,22" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 16" valign="center" halign="center" />

                         <eLabel text="Info" position="1046,693" size="71,22" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 16" valign="center" halign="center" />
                         <eLabel text="OK" position="1117,693" size="71,22" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 16" valign="center" halign="center" />
                         <eLabel text="Exit" position="1188,693" size="71,22" backgroundColor="#002f4665" transparent="0" foregroundColor="#00ffffff" zPosition="3" font="Regular; 16" valign="center" halign="center" />
                         </screen>
                        """
        Screen.__init__(self, session)
        self.session = session

        self["actions"] = ActionMap(["OkCancelActions", "DirectionActions", "EPGSelectActions", "ColorActions"], {
            "ok": self.keyOK,
            "cancel": self.keyCancel,
            "up": self.keyUp,
            "down": self.keyDown,
            "left": self.keyLeft,
            "right": self.keyRight,
            "info": self.keyInfo,
            "red": self.keyRed
        }, -1)

        self.chooseReleaseList = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
        self.chooseReleaseList.l.setItemHeight(int(50 / skinFactor))
        self.chooseReleaseList.l.setFont(0, gFont('Regular', int(28 / skinFactor)))
        self['releaseList'] = self.chooseReleaseList

        self.chooseBinList = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
        self.chooseBinList.l.setItemHeight(int(50 / skinFactor))
        self.chooseBinList.l.setFont(0, gFont('Regular', int(28 / skinFactor)))
        self['binList'] = self.chooseBinList
        
        self["info"] = Label("Info")
        self["release"] = Label(_("Release"))
        self["binFile"] = Label(_("Binary File"))
        self["reset"] = Label(_("Reset"))

        self.toggle = False
        
        self.data = []

        self.onLayoutFinish.append(self.loadRelease)

    def loadRelease(self):
        response = urllib2.urlopen(url)
        data = response.read()
        soup = BeautifulSoup(data, 'html.parser')
        release_data = soup.find_all("div", class_="d-flex flex-column flex-md-row my-5 flex-justify-center")
        self.data = []
        if release_data:
            for item in release_data:
                bin_data = []
                release = item.find("span", class_="ml-1 wb-break-all")
                release_text = release.get_text().strip().encode("utf-8") if release else None
                release_info = item.find("div", class_="markdown-body my-3")
                release_info_text = release_info.get_text().strip().encode("utf-8").replace("\n\n", "") if release_info else ""
                release_bin_data = item.find_all("li", class_="Box-row")
                if release_bin_data:
                    for bin in release_bin_data:
                        bin_text = bin.get_text().strip().encode("utf-8").replace("\n", " ") if bin else None
                        bin_url = url_git + bin.find("a", href=True)["href"].encode("utf-8") if \
                        bin.find("a", href=True)["href"] else None
                        if bin_text and bin_url:
                            bin_data.append((bin_text, bin_url))
                if release_text and bin_data:
                    self.data.append((release_text, release_info_text, bin_data))
        ext_bin = os.listdir(EXT_WLED_DESTINATION) if os.path.isdir(EXT_WLED_DESTINATION) else None
        bin_data = []
        if ext_bin:
            for find in ext_bin:
                if find.endswith(".bin"):
                    bin_destination = "%s/%s" % (EXT_WLED_DESTINATION, find)
                    bin_data.append((find, bin_destination))
            if bin_data:
                self.data.append((_("WLED unknown binary"), "", bin_data))
        if self.data:
            self.chooseReleaseList.setList(map(release_entry, self.data))
            self.showBinList()
            self.showInfo()

    def toggleList(self):
        if self.toggle:
            self.chooseBinList.selectionEnabled(0)
            self.chooseReleaseList.selectionEnabled(1)
            self.toggle = False
        else:
            self.chooseReleaseList.selectionEnabled(0)
            self.chooseBinList.selectionEnabled(1)
            self.toggle = True

    def keyUp(self):
        if self.data:
            if not self.toggle:
                self['releaseList'].up()
                self.showBinList()
            else:
                self['binList'].up()
            self.showInfo()

    def keyDown(self):
        if self.data:
            if not self.toggle:
                self['releaseList'].down()
                self.showBinList()
            else:
                self['binList'].down()
            self.showInfo()

    def keyLeft(self):
        if self.data:
            self.toggleList()
            self.showInfo()

    def keyRight(self):
        if self.data:
            self.toggleList()
            self.showInfo()

    def keyOK(self):
        if self.data:
            if not self.toggle:
                self.toggleList()
                self.showInfo()
            else:
                text = "WLED flashen ---> " + self["binList"].getCurrent()[0][1].split("/")[-1]
                self.session.openWithCallback(self.backMessage, MessageBox, text, MessageBox.TYPE_YESNO)

    def keyRed(self):
        text = _("Erase flash?\nNote: this will erase all settings stored on the ESP!")
        self.session.openWithCallback(self.backReset, MessageBox, text, MessageBox.TYPE_YESNO)

    def keyCancel(self):
#        os.system("rm /tmp/*.bin")
        self.close()

    def showInfo(self):
        if self.data:
            if not self.toggle:
                text = self["releaseList"].getCurrent()[0][1]
            else:
                text = get_info(self["binList"].getCurrent()[0][0])
            self["info"].setText(text)

    def showBinList(self):
        if self.data:
            data = self["releaseList"].getCurrent()[0][2]
            self.chooseBinList.setList(map(bin_entry, data))
            self.chooseBinList.selectionEnabled(0)

    def backReset(self, answer):
        if answer:
            cmd = "esptool.py erase_flash"
            self.session.open(Console, 'WLED flash reset', [cmd])

    def backMessage(self, answer):
        if answer:
            bin_url = self["binList"].getCurrent()[0][1]
            self.downloadBin(bin_url)

    def downloadBin(self, bin_url):
        if "https:" in bin_url:
            filedata = urllib2.urlopen(bin_url)
            datatowrite = filedata.read()
            file_destination = "/tmp/" + bin_url.split("/")[-1]
            with open(file_destination, 'wb') as f:
                f.write(datatowrite)
        else:
            file_destination = bin_url
        if os.path.isfile(file_destination):
            if "_ESP32" in file_destination and not "_bootloader" in file_destination:
                cmd = "esptool.py write_flash 0x10000 %s" % file_destination
            else:
                cmd = "esptool.py write_flash 0x0 %s" % file_destination
            self.session.open(Console, 'WLED flash', [cmd])
        else:
            error = file_destination + _(" download error!")
            self.session.open(MessageBox, error, MessageBox.TYPE_ERROR)

    def keyInfo(self):
        self.session.open(MessageBox, INFO, MessageBox.TYPE_INFO)


def release_entry(entry):
    res = [entry]
    res.append(MultiContentEntryText(pos=(int(5 / skinFactor), int(5 / skinFactor)),
                                     size=(int(390 / skinFactor), int(40 / skinFactor)),
                                     flags=RT_HALIGN_LEFT | RT_VALIGN_CENTER,
                                     font=0,
                                     text=entry[0]))

    return res


def bin_entry(entry):
    res = [entry]
    res.append(MultiContentEntryText(pos=(int(5 / skinFactor), int(5 / skinFactor)),
                                     size=(int(590 / skinFactor), int(40 / skinFactor)),
                                     flags=RT_HALIGN_LEFT | RT_VALIGN_CENTER,
                                     font=0,
                                     text=entry[0]))

    return res


def get_info(item):
    text = ""
    if "ESP8266.bin" in item:
        text = _("NodeMCU, Wemos D1 mini, ESP-12, all ESP8266 with 4Mb flash. Recommended. (The ledpin is GPIO2 in this case.)")
    elif "ESP32.bin" in item:
        text = _("All ESP32 devices (try this if the WLED-AP doesn't appear after flashing)")
    elif "ESP8266_1M_ota.bin" in item:
        text = _("ESP-01 (black PCB), most Sonoff devices, ESP8265, all ESP8266 with 1Mb flash. This binary has some interfaces disabled (Alexa, Blynk, Hue sync, Infrared) in order for wireless updates to continue working.")
    elif "ESP8266_1M_full.bin" in item:
        text = _("ESP-01 (black PCB), most Sonoff devices, ESP8265, all ESP8266 with 1Mb flash. This binary has the full feature set, but wireless updates will not work.")
    elif "ESP8266_512K.bin" in item:
        text = _("ESP-01 (blue PCB), older Sonoff devices, all ESP8266 with 512k flash. Interfaces (Alexa, Blynk, Hue sync, Infrared) disabled, no OTA. Support will not be possible in future versions.")
    elif "ESP8266_ledpin16.bin" in item:
        text = _("Custom build for 4Mb flash ESP8266 and WS2812b. LED pin is changed (default is GPIO2). LED pin 16 is useful for the QuinLed Dig Uno board. (This is GPIOY and not DY for the D to GPIO mapping, check your boards spec!)")
    elif "ESP8266_ledpin4.bin" in item:
        text = _("Custom build for 4Mb flash ESP8266 and WS2812b. LED pin is changed (default is GPIO2). LED pin 4 is useful for the QuinLed Dig Uno board. (This is GPIOY and not DY for the D to GPIO mapping, check your boards spec!)")
    elif "ESP8266_apa102.bin" in item:
        text = _("Custom build for 4Mb flash ESP8266 and APA102 LEDs (clock pin GPIO0, data GPIO2).")
    elif "ESP8266_ws2801.bin" in item:
        text = _("Custom build for 4Mb flash ESP8266 and WS2801 LEDs (clock pin GPIO0, data GPIO2).")
    elif "ESP32_ledpin16.bin" in item:
        text = _("Custom build for ESP32 and WS2812b. LED pin is changed (default is GPIO2). LED pin 16 is useful for the QuinLed Dig Uno board.")
    elif "esp32_bootloader.bin" in item:
        text = _("Not a WLED release. To be flashed to a brand new ESP32 before flashing the WLED binary itself.")
    return text


class Check:
    def __init__(self, session):
        self.session = session
        try:
            threads.deferToThread(self.readReleaseSoup, self.runMessage)
        except:
            # VTI  not support threads correctly, update check not working
            print "[WLED-Flash] Relase-Update-Check thread error!"

    def readReleaseSoup(self, callback):
        response = urllib2.urlopen(url)
        data = response.read()
        soup = BeautifulSoup(data, 'html.parser')
        release_data = soup.find("span", class_="ml-1 wb-break-all")
        release_text = None
        if release_data:
            release_text = release_data.get_text().strip().encode("utf-8") if release_data else None
        if release_text:
            # first plugin start
            if config.plugins.wledflash.update.value == "":
                config.plugins.wledflash.update.value = release_text
                config.plugins.wledflash.update.save()
                configfile.save()
                message = None
            # new version online
            elif not config.plugins.wledflash.update.value == release_text:
                print "[WLED-Flash] update online"
                message = _("WLED Release update online!")
                config.plugins.wledflash.update.value = release_text
                config.plugins.wledflash.update.save()
                configfile.save()
            # no update
            else:
                print "[WLED-Flash] no update online"
                message = None
        # read release version error
        else:
            print "[WLED-Flash] read update error"
            message = _("WLED Read update error!")
        reactor.callFromThread(callback, message)

    def runMessage(self, message=None):
        if message:
            try:
                self.session.open(MessageBox, _(message), MessageBox.TYPE_INFO)
            except:
                # modal open are allowed only from a screen which is modal
                print "[WLED-Flash] update message error!"


def sessionstart(reason, **kwargs):
    if kwargs.has_key("session") and reason == 0:
        session = kwargs["session"]
        Check(session)


def main(session, **kwargs):
    session.open(WledFlash)


def Plugins(**kwargs):
    return [PluginDescriptor(name="WLED Flash Tool v" + VERSION, description="WLED Flash Tool",
                             where=PluginDescriptor.WHERE_PLUGINMENU, icon="wledflashtool.png", fnc=main),
            PluginDescriptor(name="WLED Flash Tool v" + VERSION, description="WLED Flash Tool",
                             where=PluginDescriptor.WHERE_EXTENSIONSMENU, icon="wledflashtool.png", fnc=main),
            PluginDescriptor(name="WLED Flash Tool v" + VERSION, where=PluginDescriptor.WHERE_SESSIONSTART, fnc=sessionstart)]
